(declaim (special *CLIPBOARD* *CLIP-IDX*))
(setf *CLIPBOARD* nil)
(setf *CLIP-IDX* 0)
(setf *CLIPBOARD-UPDATED* nil)

(defun put-clip (foo)
"Push an object onto the LTKTools Clipboard.
(put-clip object)"

  (push foo *CLIPBOARD*)
  (setf *CLIP-IDX* (let ((foo (length *CLIPBOARD*)))
		     (mod (1+ *CLIP-IDX*) foo)))
  (setf *CLIPBOARD* (no-longer-than (get-config-option :clipboard-length) *CLIPBOARD*)))
       
(defun get-clip ()
"Get the currently selected object from the LTKTools Clipboard.
(get-clip)"
  (if (> (length *CLIPBOARD*) *CLIP-IDX*)
      (elt *CLIPBOARD* *CLIP-IDX*)
      nil))

(defun clipboard ()
"LTKTools Clipboard. Provides an LTK GUI interface to provide for clipboard management.
Scroll through the clipboard history to select the active clipboard item, delete clipboard
entries, and save entries to text files."
  (ltk:with-ltk ()
     (let* ((main-frame (make-instance 'ltk:frame))
 	    (button-frame (make-instance 'ltk:frame :master main-frame))
	    (display (make-instance 'ltk:scrolled-text :master main-frame))
 	    (quit-button (make-instance 'ltk:button :master button-frame :text "Quit"
				       :command (lambda ()
						  (setf ltk:*exit-mainloop* t))))
	    (next-button (make-instance 'ltk:button :master button-frame :text "Next"
				       :command (lambda ()
 						  (if (not (null *CLIPBOARD*))
						      (progn
	 						(setf *CLIP-IDX* (mod (1+ *CLIP-IDX*) (length *CLIPBOARD*)))
							(display-to-textbox (ltk:textbox display) (get-clip)))
		 				      (setf (ltk:text (ltk:textbox display)) "Clipboard is empty")))))
	    (prev-button (make-instance 'ltk:button :master button-frame :text "Prev"
					:command (lambda ()
						   (if (not (null *CLIPBOARD*))
						       (progn 
							 (setf *CLIP-IDX* (mod (1- *CLIP-IDX*) (length *CLIPBOARD*)))
							 (display-to-textbox (ltk:textbox display) (get-clip)))
						       (setf (ltk:text (ltk:textbox display)) "Clipboard is empty")))))
	    (refresh-button (make-instance 'ltk:button :master button-frame :text "Refresh"
					   :command (lambda ()
						      (if (not (null *CLIPBOARD*))
							  (display-to-textbox (ltk:textbox display) (get-clip))
							  (setf (ltk:text (ltk:textbox display)) "Clipboard is empty")))))
	    (delete-button (make-instance 'ltk:button :master button-frame :text "Delete"
					  :command (lambda ()
						     (if (and (not (null *CLIPBOARD*))
							      (>= *CLIP-IDX* 0))
							 (progn
							   (setf *CLIPBOARD* 
								 (cond
								   ((= *CLIP-IDX* 0)
								    (subseq *CLIPBOARD* 1))
								   ((= *CLIP-IDX* (1- (length *CLIPBOARD*)))
								    (subseq *CLIPBOARD* 0 (1- (length *CLIPBOARD*))))
								   (t (append 
								       (subseq *CLIPBOARD* 0 (1- *CLIP-IDX*))
								       (subseq *CLIPBOARD* (1+ *CLIP-IDX*))))))
							   (setf *CLIP-IDX* 0)
							   (if (null *CLIPBOARD*)
							       (setf (ltk:text (ltk:textbox display)) "Clipboard is empty")
							       (display-to-textbox (ltk:textbox display) (get-clip))))))))
	    (save-button (make-instance 'ltk:button :master button-frame :text "Save"
					:command (lambda ()
						   (let ((filename (ltk::ltk-popup "Enter a path/filename to save to")))
						     (with-open-file (fp (remove #\newline filename)
									 :direction :output :if-exists :overwrite
									 :if-does-not-exist :create)
						       (format fp "~A" (ltk:text (ltk:textbox display)))))))))
       (ltk:pack main-frame :side :left :expand t :fill :both)
       (ltk:pack button-frame :side :left)
       (ltk:pack display :side :left :expand t :fill :both)
       (ltk:pack quit-button :fill :x :expand t)
       (ltk:pack next-button :fill :x :expand t)
       (ltk:pack prev-button :fill :x :expand t)
       (ltk:pack save-button :fill :x :expand t)
       (ltk:pack delete-button :fill :x :expand t)
       (ltk:pack refresh-button :fill :x :expand t)

       (apply #'ltk:configure main-frame (get-config-option :clipboard-frame))
       (apply #'ltk:configure button-frame (get-config-option :clipboard-frame))
       (apply #'ltk:configure (ltk:textbox display) (get-config-option :clipboard-display))
       (apply #'ltk:configure quit-button (get-config-option :clipboard-button))
       (apply #'ltk:configure next-button (get-config-option :clipboard-button))
       (apply #'ltk:configure prev-button (get-config-option :clipboard-button))
       (apply #'ltk:configure save-button (get-config-option :clipboard-button))
       (apply #'ltk:configure delete-button (get-config-option :clipboard-button))
       (apply #'ltk:configure refresh-button (get-config-option :clipboard-button))

       (if (not (null *CLIPBOARD*))
	   (display-to-textbox (ltk:textbox display) (get-clip))
	   (setf (ltk:text (ltk:textbox display)) "Clipboard is empty")))))

