(declaim (special *LTKTOOLS-CONFIG* *LTKTOOLS-DIR*))
(setf *LTKTOOLS-CONFIG* (list (cons t t)))

(defun set-config-option (name val)
  (let ((node (assoc name *LTKTOOLS-CONFIG*)))
    (if (not (null node))
	(setf (cdr node) val)
	(nconc *LTKTOOLS-CONFIG* (list (cons name val))))))

(defun get-config-option (name)
  (cdr (assoc name *LTKTOOLS-CONFIG*)))

(defsetf get-config-option set-config-option)

;; Read in the LTKtools configuration, and set global variables
;; when this file is loaded.
(with-open-file (fp (concatenate 'string *LTKTOOLS-DIR* "/ltktools-config.lisp"))
  (setf *LTKTOOLS-CONFIG* (read fp)))

(defun ltkconfig ()
"LTKCONFIG - Simple configurator for LTKtools."
  (declaim (special *DISPLAY-LENGTH* *HISTORY-LENGTH* *AUTO-PAREN* *CLIPBOARD-LENGTH*))
  (ltk:with-ltk ()
    (let* ((frame (make-instance 'ltk:frame)) 
	   (lframe (make-instance 'ltk:frame :master frame))
	   (rframe (make-instance 'ltk:frame :master frame))
	   (label-disp-len (make-instance 'ltk:label :master lframe :text "Max. Display length: "))
	   (text-disp-len (make-instance 'ltk:entry :master rframe))
	   (label-hist-len (make-instance 'ltk:label :master lframe :text "Max. History length: "))
	   (text-hist-len (make-instance 'ltk:entry :master rframe))
	   (label-auto-paren (make-instance 'ltk:label :master lframe :text "Auto Paren Matching: "))
	   (checkbox-auto-paren (make-instance 'ltk:check-button :master rframe))
	   (label-clip-len (make-instance 'ltk:label :master lframe :text "Max. Clipboard length: "))
	   (text-clip-len (make-instance 'ltk:entry :master rframe))
	   (quit-button (make-instance 'ltk:button :master lframe :text "Cancel"
				       :command (lambda ()
						  (setf ltk:*exit-mainloop* t))))
	   (ok-button (make-instance 'ltk:button :master rframe :text "Ok"
				     :command (lambda ()
						(set-config-option :display-length (parse-integer (ltk:text text-disp-len)))
						(set-config-option :history-length (parse-integer (ltk:text text-hist-len)))
						(set-config-option :clipboard-length (parse-integer (ltk:text text-clip-len)))
						(with-open-file (fp (concatenate 'string *LTKTOOLS-DIR* "/ltktools-config.lisp")
								    :direction :output :if-exists :overwrite)
						  (format fp "~A~%" *LTKTOOLS-CONFIG*))
						(setf ltk:*exit-mainloop* t)))))
      (ltk:pack frame :side :left)
      (ltk:pack lframe :side :left)
      (ltk:pack rframe :side :left)
      (ltk:pack label-disp-len)
      (ltk:pack label-hist-len)
      (ltk:pack label-auto-paren)
      (ltk:pack label-clip-len)
      (ltk:pack text-disp-len)
      (ltk:pack text-hist-len)
      (ltk:pack checkbox-auto-paren)
      (ltk:pack text-clip-len)
      (ltk:pack quit-button)
      (ltk:pack ok-button)
      (ltk:configure text-disp-len :width 4)
      (ltk:configure text-hist-len :width 4)
      (ltk:configure text-clip-len :width 4)
      (setf (ltk:text text-disp-len) (get-config-option :display-length))
      (setf (ltk:text text-hist-len) (get-config-option :history-length))
      (setf (ltk:text text-clip-len) (get-config-option :clipboard-length)))))


