(declaim (special *addy*))
(setf *addy* nil)

(defun update-inspector-window (data-win desc-win error-win obj)
"PRIMARILY USED BY INSPECTOR
 (update-inspector-window data-window describe-window error-window object)
 Updates the given LTK widgets with information about 'object'"
  (declaim (special *addy*))
  (let* ((foo (with-output-to-string (fp)
		(describe-object obj fp)
		(cond
		  ((hash-table-p obj) 
		   (format t "~%Key    :    Value~%")
		   (maphash #'(lambda (key val)
				(format fp "~A  :  ~A~%" key val))
			    obj))))))
    (display-to-textbox (ltk:textbox data-win) obj)
    (display-to-textbox (ltk:textbox desc-win) foo)
    (setf (ltk:text error-win) (format nil "Address: ~A" *addy*))))

(defun inspector-hook (c e)
"PRIMARILY USED BY INSPECTOR
 (inspector-hook condition event)
 Function used for binding to *DEBUGGER-HOOK* by INSPECTOR "
  (declare (ignore e))
  (declaim (special *error-label* *addy* *old-addy*))
  (setf (ltk:text *error-label*) (format t "~A" c))
  (setf *addy* *old-addy*)
  (invoke-restart (find-restart 'inspector)))

(defun inspector-browse (dir value-text describe-text error-label obj)
"PRIMARILY USED BY INSPECTOR
 (inspector-browse direction value-textbox describe-textbox error-label object)
 Used for updating current address information, and LTK widget contents when
 navigating a Lisp form."
  (declaim (special *addy* *old-addy* *error-label*))
  (cond
    ((eq dir :up)
     (let ((*debugger-hook* #'inspector-hook))
       (with-simple-restart (inspector "Inspector Restart")
	 (setf *old-addy* (copy-list *addy*))
	 (setf *addy* (cond
			((null *addy*) nil)
			(t (subseq *addy* 0 (1- (length *addy*))))))
	 (apply #'ref obj *addy*)))
     (update-inspector-window value-text describe-text error-label
			      (apply #'ref obj *addy*)))
    ((eq dir :down)
     (if (and (not (atomic (apply #'ref obj *addy*)))
	      (not (null (apply #'reflen obj *addy*)))
	      (> (apply #'reflen obj *addy*) 0))
	 (let ((*debugger-hook* #'inspector-hook))
	   (with-simple-restart (inspector "Inspector Restart")
	     (setf *old-addy* (copy-list *addy*))
	     (setf *addy* (append *addy* (list 0)))
	     (update-inspector-window value-text describe-text error-label
				      (apply #'ref obj *addy*))))))
    ((eq dir :right)
     (let ((*debugger-hook* #'inspector-hook))
       (if (integerp (car (last *addy*)))
	   (with-simple-restart (inspector "Inspector Restart")
	     (setf *old-addy* (copy-list *addy*))
	     (setf (car (last *addy*)) (mod (1+ (car (last *addy*))) 
					    (apply #'reflen obj (subseq *addy* 0 (1- (length *addy*))))))
	     (apply #'ref obj *addy*)))
       (update-inspector-window value-text describe-text error-label
				(apply #'ref obj *addy*))))
    ((eq dir :left)
     (let ((*debugger-hook* #'inspector-hook))
       (with-simple-restart (inspector "Inspector Restart")
	 (setf *old-addy* (copy-list *addy*))
	 (setf (car (last *addy*)) (mod (1- (car (last *addy*))) 
					(apply #'reflen obj (subseq *addy* 0 (1- (length *addy*))))))
	 (apply #'ref obj *addy*)))
     (update-inspector-window value-text describe-text error-label
			      (apply #'ref obj *addy*)))))

(defun skiperror (c e)
"A simple *DEBUGGER-HOOK* function to skip all errors, resorting to a
 SKIP restart, as established by WITH-SIMPLE-RESTART.
 Example:
   (with-simple-restart (skip restart-name-string)
     (let ((*debugger-hook* #'skiperror))
        ...
        ...))"
(declare (ignore c e))
(invoke-restart (find-restart 'skip)))

(defun inspector (&optional (obj nil) &key (allow-edit nil))
"LTKTools Inspector - Navigate Common Lisp data with an LTK GUI
Use the buttons, or the arrow keys to navigate through lists,
arrays, hash-tables, packages and other data.
(inspector (&optional data-object) &key (allow-edit nil))
Specifying :allow-edit T turns on an editing mode where simple
editing operations such as deletion, pasting from the clipboard,
and insertion. If no object is specified for browsing, the 
inspector will pop a dialog box to obtain a Lisp form or variable
to inspect."
(declaim (special *old-addy* *addy*))
  (if (null obj)
      (let ((*debugger-hook* #'skiperror))
	(with-simple-restart (skip "Skip Eval")
	  (setf obj (eval (read-from-string (let ((foo (ltk::ltk-popup "Enter a form or variable name to inspect")))
					      (if (equal foo "")
						  "nil"
						  foo))))))))
  (if (not (null obj))
      (let ((*old-addy* nil)
	    (*addy* nil)
	    (*error-label* nil))
	(declaim (special *old-addy* *addy*))
	(ltk:with-ltk (:debug 0)
	  (let* ((outer-frame (make-instance 'ltk:frame))
		 (button-frame (make-instance 'ltk:frame :master outer-frame))
		 (text-frame (make-instance 'ltk:frame :master outer-frame))
		 (text-boxes (list (make-instance 'ltk:scrolled-text :master text-frame)
				   (make-instance 'ltk:scrolled-text :master text-frame)))
		 (error-label (make-instance 'ltk:label :master text-frame :text ""))
		 (value-text (first text-boxes))
		 (describe-text (second text-boxes))
		 (buttons (list 
			   (make-instance 'ltk:button :master button-frame :text "Quit" 
					  :command (lambda ()
						     (setf ltk:*exit-mainloop* t)))
			   (make-instance 'ltk:button :master button-frame :text "Up"
					  :command (lambda ()
						     (inspector-browse :up value-text describe-text error-label obj)))
			   (make-instance 'ltk:button :master button-frame :text "Down"
					  :command (lambda ()
						     (inspector-browse :down value-text describe-text error-label obj)))
			   (make-instance 'ltk:button :master button-frame :text "Left"
					  :command (lambda ()
						     (inspector-browse :left value-text describe-text error-label obj)))
			   (make-instance 'ltk:button :master button-frame :text "Right"
					  :command (lambda ()
						     (inspector-browse :right value-text describe-text error-label obj)))
			   (make-instance 'ltk:button :master button-frame :text "Copy"
					  :command (lambda ()
						     (put-clip (apply #'ref obj *addy*))))
			   (make-instance 'ltk:button :master button-frame :text "Search"
					  :command (lambda ()
						     (let ((n (search (list
								       (read-from-string
									(ltk::ltk-popup "Form to search for:")))
								      (let ((foo (apply #'ref obj *addy*)))
									(if (atomic foo)
									    (let ((parent (subseq *addy* 0 (1- (length *addy*)))))
									      (setf *addy* parent)
									      (expand-to-list (apply #'ref obj parent)))
									    (expand-to-list (apply #'ref obj *addy*)))))))
						       (if (not (null n))
							   (nconc *addy* (list n))
							   (nconc *addy* (list 0)))
						       (update-inspector-window value-text describe-text error-label
										(apply #'ref obj *addy*))))))))
	    (if allow-edit
		(nconc buttons 
		       (list
			(make-instance 'ltk:button :master button-frame :text "Cut"
				       :command (lambda ()
						  (put-clip (ref obj *addy*))
						  (if (apply #'delref obj *addy*)
						      (progn
							(setf *addy* (subseq *addy* 0 (1- (length *addy*))))
							(update-inspector-window value-text describe-text error-label
										 (apply #'ref obj *addy*)))				 
						      (setf (ltk:text error-label) "Cannot cut this element."))))
			(make-instance 'ltk:button :master button-frame :text "Paste"
				       :command (lambda ()
						  (apply #'sref obj (append *addy* (list (get-clip))))
						  (update-inspector-window value-text describe-text error-label
									   (apply #'ref obj *addy*))))
			(make-instance 'ltk:button :master button-frame :text "Delete"
				       :command (lambda ()
						  (if (apply #'delref obj *addy*)
						      (progn
							(setf *addy* (subseq *addy* 0 (1- (length *addy*))))
							(update-inspector-window value-text describe-text error-label
										 (apply #'ref obj *addy*)))
						      (setf (ltk:text error-label) "Cannot delete this element."))))
			(make-instance 'ltk:button :master button-frame :text "Insert"
				       :command (lambda ()
						  (if (apply #'insref obj *addy*)
						      (update-inspector-window value-text describe-text error-label
									       (apply #'ref obj *addy*))
						      (setf (ltk:text error-label) "Cannot insert an element here.")))))))
	    
	    (ltk:pack outer-frame :fill :both :expand t)
	    (ltk:pack button-frame :side :left)
	    (dolist (b buttons)
	      (ltk:pack b :expand t :fill :x))
	    (ltk:pack text-frame :fill :both :expand t)
	    (ltk:pack value-text :fill :both :expand t)
	    (ltk:pack describe-text :fill :both :expand t)
	    (ltk:pack error-label :fill :x :expand t)
	    (setf *error-label* error-label)

	    (apply #'ltk:configure outer-frame (get-config-option :inspector-frame))
	    (apply #'ltk:configure button-frame (get-config-option :inspector-frame))
	    (apply #'ltk:configure text-frame (get-config-option :inspector-frame))

	    (let ((opts (get-config-option :inspector-button)))
	      (dolist (b buttons)
		(apply #'ltk:configure b opts)))

	    (apply #'ltk:configure (ltk:textbox value-text) (get-config-option :inspector-value))
	    (apply #'ltk:configure (ltk:textbox describe-text) (get-config-option :inspector-desc))

	    (ltk:bind (ltk:textbox value-text) (get-config-option :inspector-key-parse)
		      #'(lambda (evt)
			  (declare (ignore evt))
			  (if allow-edit
			      (let ((*debugger-hook* #'skiperror))
				(with-simple-restart (skip "Skip Errors")
				  (let ((foo (read-from-string (ltk:text (ltk:textbox value-text)))))
				    (apply #'sref obj (append *addy* (list foo)))))))
			  (update-inspector-window value-text describe-text error-label
						   (apply #'ref obj *addy*))))
	    (ltk:bind (ltk:textbox value-text) (get-config-option :inspector-key-up)
		      (lambda (evt)
			(declare (ignore evt))
			(inspector-browse :up value-text describe-text error-label obj)))
	    (ltk:bind (ltk:textbox value-text) (get-config-option :inspector-key-down)
		      (lambda (evt)
			(declare (ignore evt))
			(inspector-browse :down value-text describe-text error-label obj)))
	    (ltk:bind (ltk:textbox value-text) (get-config-option :inspector-key-right)
		      (lambda (evt)
			(declare (ignore evt))
			(inspector-browse :right value-text describe-text error-label obj)))
	    (ltk:bind (ltk:textbox value-text) (get-config-option :inspector-key-left)
		      (lambda (evt)
			(declare (ignore evt))
			(inspector-browse :left value-text describe-text error-label obj)))
	    (ltk:bind (ltk:textbox value-text) (get-config-option :inspector-key-copy)
		      (lambda (evt)
			(declare (ignore evt))
			(put-clip (apply #'ref obj *addy*))))
	    (if allow-edit
		(ltk:bind (ltk:textbox value-text) (get-config-option :inspector-key-paste)
			  (lambda (evt)
			    (declare (ignore evt))
			    (apply #'sref obj (append *addy* (list (get-clip))))
			    (update-inspector-window value-text describe-text error-label
						     (apply #'ref obj *addy*)))))
	    
	    (update-inspector-window value-text describe-text error-label obj))))))
  
