(declaim (special *LTKTOOLS-DIR* *COMPILE-FILES* *DATA-FILES* *LOADER-NAME*))
(setf *COMPILE-FILES* '("ltk-x" "utils" "config" "clipboard" "profiler" "inspector" "spindle" "console"))
(setf *DATA-FILES* '("ltkconsole-buttons" "ltktools-config"))
#+cmu (setf *LOADER-NAME* "ltk-tools-library")
#+sbcl (setf *LOADER-NAME* "ltk-tools")

#+cmu (require 'ltk)
#+sbcl (require 'ltk)
#+clisp (require "ltk")

  (defun de-tokenize-string (str token)
      (let ((tmp "")
	  (ret-list '()))
      (dotimes (n (length str) (append ret-list (list tmp)))
	(if (eq (elt str n) token)
	    (progn
	      (setf ret-list (append ret-list (list tmp)))
	      (setf tmp ""))
	    (setf tmp (concatenate 'string tmp (princ-to-string (elt str n))))))))

  (defun copy-text-file (src dest)
    (with-open-file (src-fp src :direction :input)
      (with-open-file (dest-fp dest :direction :output :if-exists :overwrite :if-does-not-exist :create)
	(do ((l (read-line src-fp) (read-line src-fp nil 'eof)))
	    ((eq l 'eof) nil)
	  (format dest-fp "~A~%" l)))))

(defun install ()
  ;; Recieve the installation directory from the user
  ;; We give them as many times as they need to get it right.
#+cmu (setf *LTKTOOLS-DIR* "/usr/local/lib/cmucl/lib/subsystems")
#+sbcl (setf *LTKTOOLS-DIR* "/usr/lib/sbcl/ltk-tools")
  (format t "The installation directory is set to [~A].~%" *LTKTOOLS-DIR*)
  (do ((exit-now nil (progn
		       (format t "Is ~A ok (T/NIL) " *LTKTOOLS-DIR*)
		       (force-output)
		       (read))))
      (exit-now nil)
    (format t "Enter the directory to install to: ")
    (force-output)
    (let ((foo (read-line)))
      (if (not (equal foo ""))
	  (setf *LTKTOOLS-DIR* foo))))

  ;; Trim any trailing /
  (if (eq (elt *LTKTOOLS-DIR* (1- (length *LTKTOOLS-DIR*))) #\/)
      (setf *LTKTOOLS-DIR* (subseq *LTKTOOLS-DIR* 0 (1- (length *LTKTOOLS-DIR*)))))

  ;; Make sure directory exists
  (do ((exit-now (directory *LTKTOOLS-DIR*)
		 (directory *LTKTOOLS-DIR*)))
      (exit-now nil)
    (format t "Directory does not exist. Please create it.~%Press ENTER when ready.~%")
    (read-line))

  ;; Create the ltk-tools.lisp source file
  (format t "Creating ltk-tools source file.~%")
  (with-open-file (fp (concatenate 'string *LTKTOOLS-DIR* "/" *LOADER-NAME* ".lisp") 
		      :direction :output 
		      :if-exists :overwrite
		      :if-does-not-exist :create)
    (let ((*print-readably* t)
	  (*print-pretty* t)
	  (loader-code (append `((declaim (special *LTKTOOLS-DIR*))
				 (require 'ltk)
				 (setf *LTKTOOLS-DIR* ,*LTKTOOLS-DIR*)
				 (setf ltk:*debug-tk* nil))
			       (map 'list (lambda (x)
					    `(load (concatenate 'string *LTKTOOLS-DIR* "/" ,x)))
				    *compile-files*))))
      (dolist (L loader-code)
	(print L fp))))

  ;; Copy all install files
  (dolist (file (append *COMPILE-FILES* *DATA-FILES*))
    (format t "Copying file ~A.lisp to ~A~A.lisp~%" file *LTKTOOLS-DIR* file)
    (copy-text-file (concatenate 'string file ".lisp")
		    (concatenate 'string *LTKTOOLS-DIR* "/" file ".lisp")))
  
  ;; Change directory
#+cmu  (setf (ext:default-directory) *LTKTOOLS-DIR*)
#+sbcl (setf *DEFAULT-PATHNAME-DEFAULTS* (make-pathname :directory *LTKTOOLS-DIR*))

  ;; Compile the source
  (dolist (file *compile-files*)
    (load file)
    (compile-file file))

  (compile-file *LOADER-NAME*)

  ;; Remove the source
  (dolist (file *compile-files*)
    (delete-file (concatenate 'string file ".lisp")))

  (delete-file (concatenate 'string *LOADER-NAME* ".lisp"))

  (format t "~%Installation complete. Load the compiled ltk-tools file from the~%installation directory to load LTKtools~%"))

(defun uninstall ()
  (declaim (special *LTKTOOLS-DIR*))
  (let ((ext
#+cmu ".x86f"
#+sbcl ".fasl"
         ))
    (delete-file (concatenate 'string *LTKTOOLS-DIR* "/" *LOADER-NAME* ext))
    (dolist (f *COMPILE-FILES*)
      (format t "Removing ~A~%" (concatenate 'string *LTKTOOLS-DIR* "/" f ext))
      (delete-file (concatenate 'string *LTKTOOLS-DIR* "/" f ext)))
    (dolist (f *DATA-FILES*)
      (format t "Removing ~A~%" (concatenate 'string *LTKTOOLS-DIR* "/" f ".lisp"))
      (delete-file (concatenate 'string *LTKTOOLS-DIR* "/" f ".lisp")))))
