(declaim (special *PROF-RESULT*))

(defun profhook (c e)
"PROFHOOK - *DEBUGGER-HOOK* function for use by profiler"
  (declare (ignore e))
  (setf (ltk:text (ltk:textbox *PROF-RESULT*)) 
	(format nil "DEBUG: ~A~%" c))
  (invoke-restart (find-restart 'profeval)))

(defun profiler ()
"LTKTools Profiler - Provide an LTK GUI interface to Marty Hall's simple metering tools."
  (ltk:with-ltk (:debug 0)
    (let* ((frame (make-instance 'ltk:frame))
	   (lframe (make-instance 'ltk:frame :master frame))
	   (rframe (make-instance 'ltk:frame :master frame))
	   (label-functions (make-instance 'ltk:label :text "Function names:" :master lframe))
	   (text-functions (make-instance 'ltk:text :master lframe))
	   (label-code (make-instance 'ltk:label :text "Body of code:" :master lframe))
	   (text-code (make-instance 'ltk:scrolled-text :master lframe))
	   (label-result (make-instance 'ltk:label :text "Results:" :master rframe))
	   (text-result (make-instance 'ltk:scrolled-text :master rframe))
	   (button-profile (make-instance 'ltk:button :text "Profile" :master lframe
					  :command (lambda ()
						     (let ((*debugger-hook* #'profhook))
						       (with-simple-restart (profeval "Profiler Eval")
							 (let* ((funcs (read-from-string
									(concatenate 'string "(" (ltk:text text-functions) ")")))
								(code (read-from-string
								       (ltk:text (ltk:textbox text-code))))
								(realt (get-internal-real-time))
								(runt (get-internal-run-time))
								(consd (get-bytes-consed)))
							   (multiple-value-bind (ret-val counts)
							       (eval `(without-package-locks
								       (with-function-call-count ,funcs 
									 ,code)))
							     (let ((foo (map 'list #'(lambda (x y)
										       (cons x y))
									     funcs counts))
								   (fp (make-string-output-stream)))
							       (setf (ltk:text (ltk:textbox text-result))
								     (progn
								       (format fp "Elapsed real time : ~A~%" (- (get-internal-real-time)
														realt))
								       (format fp "Elapsed run time  : ~A~%" (- (get-internal-run-time)
														runt))
								       (format fp "Total bytes consed: ~A~%" (- (get-bytes-consed)
														consd))
								       (format fp "Return value: ~A~%" ret-val)
								       (format fp "Function call counts:~%")
								       (dolist (n foo (get-output-stream-string fp))
									 (format fp "~A  =  ~A~%" (car n) (cdr n)))))))))))))
	     (button-quit (make-instance 'ltk:button :text "Quit" :master rframe
					 :command (lambda ()
						    (setf ltk:*exit-mainloop* t)))))
      (setf *PROF-RESULT* text-result)
      (ltk:pack frame :expand t :fill :both)
      (ltk:pack lframe :side :left :expand t :fill :both)
      (ltk:pack rframe :side :left :expand t :fill :both)
      (apply #'ltk:configure frame (get-config-option :profiler-frame))
      (apply #'ltk:configure lframe (get-config-option :profiler-frame))
      (apply #'ltk:configure rframe (get-config-option :profiler-frame))

      (ltk:pack label-functions)
      (apply #'ltk:configure label-functions (get-config-option :profiler-label))

      (ltk:pack text-functions :expand t :fill :both)
      (apply #'ltk:configure text-functions (get-config-option :profiler-func))

      (ltk:pack label-code)
      (apply #'ltk:configure label-code (get-config-option :profiler-label))      

      (ltk:pack text-code :expand t :fill :both)
      (apply #'ltk:configure (ltk:textbox text-code) (get-config-option :profiler-code))

      (ltk:pack button-profile)
      (apply #'ltk:configure button-profile (get-config-option :profiler-button))

      (ltk:pack label-result)
      (apply #'ltk:configure label-result (get-config-option :profiler-label))

      (ltk:pack text-result :expand t :fill :both)
      (apply #'ltk:configure (ltk:textbox text-result) (get-config-option :profiler-result))

      (ltk:pack button-quit)
      (apply #'ltk:configure button-quit (get-config-option :profiler-button))

      (ltk:bind text-functions (get-config-option :profiler-key-paste) 
		(lambda (evt)
		  (declare (ignore evt))
		  (display-to-textbox text-functions (get-clip))))
      (ltk:bind (ltk:textbox text-code) (get-config-option :profiler-key-paste)
		(lambda (evt)
		  (declare (ignore evt))
		  (display-to-textbox (ltk:textbox text-code) (get-clip))))
      (ltk:bind (ltk:textbox text-result) (get-config-option :profiler-key-copy)
		(lambda (evt)
		  (declare (ignore evt))
		  (put-clip (ltk:text (ltk:textbox text-result)))))
      (ltk:bind (ltk:textbox text-code) (get-config-option :profiler-key-copy)
		(lambda (evt)
		  (declare (ignore evt))
		  (put-clip (ltk:text (ltk:textbox text-code)))))
      (ltk:bind text-functions (get-config-option :profiler-key-copy)
		(lambda (evt)
		  (declare (ignore evt))
		  (put-clip (ltk:text text-functions)))))))


;;;===========================================================================
;;;===========================================================================
;;; Some simple tracing/metering utilities. marty_hall@jhuapl.edu.
;;;
;;; With-Function-Call-Count: Takes a list of function names and a body of code
;;; ========================  and returns two values: (A) the normal return
;;;                           value of the body of code and (B) a list of the
;;;                           number of times the functions were called during
;;;                           the execution of the Body.
;;;                           > (With-Function-Call-Count (F1 F2 F3)
;;;                               (Foo))
;;;                           Foo-Result   ; return result of (Foo)
;;;                           (X Y Z)      ; F1 was called X times, F2 Y
;;;                                        ; times and F3 Z times while
;;;                                        ; executing (Foo).
;;;
;;; Time-Form: Takes a single form and optionally an integer N (default 20). It
;;; =========  runs the form N times and prints out the average time. Useful
;;;            for timing very small, fast functions when the time-step of
;;;            the builtin TIME function is too coarse.
;;;            > (Time-Form (Foo))     ; Call (Foo) 20 times + print avg time
;;;            > (Time-Form (Bar) 100) ; Call (Bar) 100 times + print avg time
;;;
;;; Breakon: Takes a function name and changes the function to enter BREAK on
;;; =======  entry to that function. Lets you do a backtrace and other
;;;          options your debugger gives you.
;;;          > (Breakon 'F1)
;;;          > (Foo)           ; assume F1 gets called when Foo executes
;;;          Break: `Breakon' specified for function F1.
;;;          <Debugger Options>
;;;
;;; Unbreakon: Returns a function to state it was in before Breakon was called.
;;; =========
;;;
;;; Make-Timer (and Run-Timer and Timer-Results):
;;; ============================================
;;;   This is used in the rare instances when you need to time something that
;;;   is near to or faster than then internal clock, but where you can't run
;;;   it multiple times in a row, as with Time-Form. An example is a function
;;;   where certain parameters need to get reset (or a table cleared) after
;;;   each execution, but you don't want to include the time of resetting the
;;;   parameters (or clearing the table) in your benchmark. For instance,
;;;   suppose Foo is the function you want to time, and Reset-Foo is the
;;;   procedure that clears out the table or resets the parameters Foo
;;;   needs. In such a case, you would do the following, assuming '> ' is
;;;   the Lisp prompt:
;;;   
;;;   > (setq Timer-1 (Make-Timer (Foo)))
;;;   > (loop repeat <Num Trials> do
;;;      (Run-Timer Timer-1)
;;;      (Reset-Foo) )
;;;   > (Timer-Results Timer-1)
;;;   (Foo) was executed <Num Trials> times, taking an average of xxx seconds.
;;;
;;; Without-GC: Lucid-specific macro that executes a body of code with the
;;; ==========  ephemeral and dynamic garbage collectors turned off.
;;;
;;; With-Metering: On Lucid Common Lisp, this is Lucid-specific macro that
;;; =============  executes a body of code with the Lucid metering system
;;;                turned on, so that you get a printout of the percentage
;;;                of time spent in various subfunctions. Adapted from code
;;;                by Paul McNamee.
;;;
;;;                On Harlequin, this is a LispWorks-specific macro that
;;;                executes a body of code within the Harlequin profiler.
;;;
;;;                On Allegro Common Lisp, this is an ACL-specific macro that
;;;                executes a body of code within the Allegro profiler.
;;;
;;;                In all cases, see the doc string for examples and details.
;;; 
;;; 1993-95 Marty Hall. marty_hall@jhuapl.edu
;;; The Johns Hopkins University Applied Physics Lab
;;;
;;; This code may be freely copied and used without restriction.
;;;===========================================================================
;;;===========================================================================


;;;===========================================================================
;;; Takes a list of function names and a body of code, and returns a list of
;;; the number of times the functions were called during the execution of the
;;; Body. See the doc string for more details. The UNWIND-PROTECT is to make
;;; sure the call-count gets reset even if the Body of code crashes. Also,
;;; this works for either regular or generic functions, but there is no way
;;; to specify that only one particular method of a generic function gets
;;; counted. Also (to risk stating the obvious), this will not work for
;;; counting macros or INLINEd functions. 3/93 Marty Hall.

(defmacro With-Function-Call-Count (Function-Name-List &body Body)
  "Takes a list of function names and a body of code, and returns two values:
   (A) the normal return value of the body of code and (B) a list of the
   number of times the functions were called during the execution of the Body.
   Eg:

   (With-Function-Call-Count (Speed Latitude Longitude)
     (Make-Top-Level-Display)
     (Make-MAD-Display))

   returns (867 651 651) as the secondary value, indicating SPEED was called
   867 times, and LATITUDE and LONGITUDE 651 times each during the execution
   of the top-level and MAD displays."
  (let ((Call-Count-Variable (gensym "CALL-COUNT-"))
	(Return-Value (gensym "RETURN-VALUE-")))
    `(let (,Call-Count-Variable ,Return-Value)
       (unwind-protect
	   (progn
	     (mapc #'Make-Function-Countable ',Function-Name-List)
	     (setq ,Return-Value (progn ,@Body))
	     (setq ,Call-Count-Variable
		   (mapcar #'(lambda (Function-Name)
			       (get Function-Name :Call-Count))
			   ',Function-Name-List))
	     (values ,Return-Value ,Call-Count-Variable) )
	 (mapc #'Make-Function-Uncountable ',Function-Name-List)))))

;;;===========================================================================
;;; Changes a function from its normal version to one that counts how often it
;;; is called. Should only be used temporarily. Also note this won't work for
;;; recursive routines without the addition of Make-Function-Countable,
;;; because the internal calls go to the non-counting version.
;;; This is normally called from With-Function-Call-Count, not directly.
;;; 3/93 Marty Hall

(defun Countable-Function (Function-Name)
  "Takes a function NAME and returns a function OBJECT that does what #'NAME
   did, except also keeps track of the number of times it has been called"
  (let ((Function (symbol-function Function-Name)))
    (setf (get Function-Name :Call-Count) 0)
    (setf (get Function-Name :Non-Counting-Function) Function)
    #'(lambda (&rest Args)
	(incf (the fixnum (get Function-Name :Call-Count)))
	(apply Function Args))))

;;;===========================================================================
;;; Makes function countable. Generally accessed via With-Function-Call-Count.
;;; 3/93 Marty Hall.

(defun Make-Function-Countable (Function-Name)
  "Given a function name changes it into equivalent version that counts
   function calls"
  (setf (symbol-function Function-Name)
	(Countable-Function Function-Name)))

;;;===========================================================================
;;; Undoes the above.  3/93 Marty Hall

(defun Make-Function-Uncountable (Function-Name)
  "Returns the function to its original (non-counting) state"
  (let ((Original (get Function-Name :Non-Counting-Function)))
    (cond      
      (Original
       (setf (symbol-function Function-Name) Original)
       (remf (symbol-plist Function-Name) :Non-Counting-Function)
       (remf (symbol-plist Function-Name) :Call-Count)
       Original)
      (t
       (format nil "~%Function ~S wasn't countable to begin with: unchanged."
	           Function-Name)))))

