(defun show-threads (lb threads)
"(show-threads list-box thread-list)
Displays the names of the threads given in thread-list to the ltk:listbox widget list-box."
  (ltk:listbox-clear lb)
  (ltk:listbox-append lb (map 'list #'(lambda (x) (get-name-of-thread x)) threads)))

(defun spindle ()
"Spindle - LTKTools thread manager
Provides an LTK GUI interface for managing Lisp threads. View running threads, and kill them.
On some platforms, pausing and re-starting threads is also supported."
  (ltk:with-ltk (:debug 0)
    (let* ((threads (get-list-of-threads))
	   (frame (make-instance 'ltk:frame))
	   (bframe (make-instance 'ltk:frame :master frame))
	   (tframe (make-instance 'ltk:frame :master frame))
	   (proc-box (make-instance 'ltk:scrolled-listbox :master tframe))
	   (quit-button (make-instance 'ltk:button :master bframe :text "Quit"
				       :command (lambda ()
						  (setf ltk:*exit-mainloop* t))))
	   (kill-button (make-instance 'ltk:button :master bframe :text "Kill"
				       :command (lambda ()
						  (declaim (special threads))
						  (let ((thread (elt threads
								     (car (ltk:listbox-get-selection 
									   (ltk:listbox proc-box))))))
						  (kill-thread thread)
						  (sleep 1)
						  (setf threads (get-list-of-threads))
						  (show-threads (ltk:listbox proc-box) threads)))))
	   (refresh-button (make-instance 'ltk:button :master bframe :text "Refresh"
					  :command (lambda ()
						     (declaim (special threads))
						     (setf threads (get-list-of-threads))
						     (show-threads (ltk:listbox proc-box) threads))))
	   (pause-button (make-instance 'ltk:button :master bframe :text "Pause"
					:command (lambda ()
						   (declaim (special threads))
						   (let ((thread (elt threads
								      (car (ltk:listbox-get-selection
									    (ltk:listbox proc-box))))))
						     (pause-thread thread)))))
	   (enable-button (make-instance 'ltk:button :master bframe :text "Enable"
					 :command (lambda ()
						    (declaim (special threads))
						    (let ((thread (elt threads
								       (car (ltk:listbox-get-selection
									     (ltk:listbox proc-box))))))
						      (enable-thread thread)))))
	   (desc-text (make-instance 'ltk:scrolled-text :master tframe)))

	(ltk:pack frame :expand t :fill :both)
	(ltk:pack bframe :side :right)
	(ltk:pack tframe :side :right :expand t :fill :both)
	(ltk:pack quit-button :expand t :fill :x)
	(ltk:pack kill-button :expand t :fill :x)
	(ltk:pack refresh-button :expand t :fill :x)
#-sbcl	(ltk:pack pause-button :expand t :fill :x)
#-sbcl	(ltk:pack enable-button :expand t :fill :x)
	(ltk:pack proc-box :expand t :fill :both)
	(ltk:pack desc-text :expand t :fill :both)

	(apply #'ltk:configure frame (get-config-option :spindle-frame))
	(apply #'ltk:configure bframe (get-config-option :spindle-frame))
	(apply #'ltk:configure tframe (get-config-option :spindle-frame))
	(apply #'ltk:configure quit-button (get-config-option :spindle-button))
	(apply #'ltk:configure kill-button (get-config-option :spindle-button))
	(apply #'ltk:configure refresh-button (get-config-option :spindle-button))
#-sbcl	(apply #'ltk:configure pause-button (get-config-option :spindle-button))
#-sbcl	(apply #'ltk:configure enable-button (get-config-option :spindle-button))
	(apply #'ltk:configure (ltk:listbox proc-box) (get-config-option :spindle-list))
	(apply #'ltk:configure (ltk:textbox desc-text) (get-config-option :spindle-text))


	(ltk:bind (ltk:listbox proc-box) "<<ListboxSelect>>"
       	  (lambda (evt)
		    (declare (ignore evt))
		    (declaim (special threads))
		    (setf (ltk:text (ltk:textbox desc-text)) 
			  (with-output-to-string (fp)
			    (describe-object (elt threads 
						  (car (ltk:listbox-get-selection (ltk:listbox proc-box))))
					     fp)))))
	(show-threads (ltk:listbox proc-box) threads))))



      
	   